//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/

package hello;

import org.omg.CORBA.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import orbasec.corba.*;
import orbasec.corba.Opaque;
import org.omg.Security.*;

public class Client
{
	static ORB orb;
	static BOA boa;
	static orbasec.SecLev2.Current current;

	
	// Object reference with different DelegationDirective Policies on them
	static Hello  hello_no_delegate, hello_delegate;
	static Relay  relay_no_delegate, relay_delegate;
	
	// Intended Security Names for the Client and the Relay that
	// the Hello server should verify;
	static byte[] our_security_name, relay_security_name;
	
	private static Hello get_hello()
	{
    	// 
		// Get "hello" object
    	//
		System.out.println("Getting Hello Reference");
    	String ref = null;
    	try {
			String refFile = "Hello.ref";
			FileInputStream file = new FileInputStream(refFile);
			BufferedReader in =
				new BufferedReader(new InputStreamReader(file));
			ref = in.readLine();
			file.close();
    	} catch(IOException ex) {
			System.err.println("Can't read from `" +
				   ex.getMessage() + "'");
			System.exit(1);
    	}

    	org.omg.CORBA.Object obj = orb.string_to_object(ref);
    	if(obj == null)
			throw new RuntimeException();


		Hello hello = HelloHelper.narrow(orb.string_to_object(ref));
		if(hello == null)
			throw new RuntimeException();
		return hello;
	}
	
	private static Relay get_relay()
	{
    	// 
		// Get "relay" object
    	//
		System.out.println("Getting Relay Reference");
    	String ref = null;
    	try {
			String refFile = "Relay.ref";
			FileInputStream file = new FileInputStream(refFile);
			BufferedReader in =
				new BufferedReader(new InputStreamReader(file));
			ref = in.readLine();
			file.close();
    	} catch(IOException ex) {
			System.err.println("Can't read from `" +
				   ex.getMessage() + "'");
			System.exit(1);
    	}

    	org.omg.CORBA.Object obj = orb.string_to_object(ref);
    	if(obj == null)
			throw new RuntimeException();


		Relay relay = RelayHelper.narrow(orb.string_to_object(ref));
		if(relay == null)
			throw new RuntimeException();
		return relay;
	}
	
	private static Relay 
	get_relay(Relay relay, org.omg.Security.DelegationDirective deldir)
		throws org.omg.CORBA.PolicyManagerPackage.InvalidPolicies,
			   org.omg.CORBA.ORBPackage.InvalidName
	{
		//
		// Modify Object Invocation Policies
		//
		org.omg.CORBA.Policy[] policies;
		policies = new org.omg.CORBA.Policy[1];

		policies[0] = 
		orbasec.SL2.create_delegation_directive_policy(deldir);

		//
		// Set the policy on the hello_2 Object reference
		//
		Relay relay1 =
			RelayHelper.narrow(
				relay._set_policy_overrides(
					policies, 
					org.omg.CORBA.SetOverrideType.ADD_OVERRIDE));
		return relay1;
	}
	
	private static Hello 
	get_hello(Hello hello, org.omg.Security.DelegationDirective deldir)
		throws org.omg.CORBA.PolicyManagerPackage.InvalidPolicies,
			   org.omg.CORBA.ORBPackage.InvalidName
	{
		//
		// Modify Object Invocation Policies
		//
		org.omg.CORBA.Policy[] policies;
		policies = new org.omg.CORBA.Policy[1];

		policies[0] = 
		orbasec.SL2.create_delegation_directive_policy(deldir);

		//
		// Set the policy on the hello_2 Object reference
		//
		Hello hello1 =
			HelloHelper.narrow(
				hello._set_policy_overrides(
					policies, 
					org.omg.CORBA.SetOverrideType.ADD_OVERRIDE));
		return hello1;
	}
	
	private static void init(String args[])
		throws org.omg.CORBA.ORBPackage.InvalidName
	{
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			orb = orbasec.SL2.orb();
			boa = orbasec.SL2.boa();

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		current = 
				orbasec.SecLev2.CurrentHelper.narrow(
					orb.resolve_initial_references("SecurityCurrent"));
	}

	private static void authenticate()
	{
    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = "Kerberos";
			int	    method    = 0;
			byte	security_name[] = 
//@@@@ Change this to a valid kerberos principal!
				    Opaque.encodeKerberosName(
						"marge@MYREALM.COM"
					).getEncoding();

			byte	auth_data[] =
			        (
//@@@@ Change this to principal's password!
				     "password=\"mypassword\"\n" +
//@@@@ Setting delegation capability on	
					 "delegation=true\n" +
//@@@@ Using a memory credentials cache
				     "cache_name=MEMORY:0\n" +
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();

			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds,
				continuation_data,
				auth_specific_data
			);

	}
	
	private static void setup_references()
		throws  org.omg.CORBA.PolicyManagerPackage.InvalidPolicies,
		        org.omg.CORBA.ORBPackage.InvalidName
	{
		Hello hello = get_hello();
		Relay relay = get_relay();

		hello_delegate = 
			get_hello(hello,
					  org.omg.Security.DelegationDirective.Delegate);
		hello_no_delegate = 
			get_hello(hello,
					  org.omg.Security.DelegationDirective.NoDelegate);
		relay_delegate = 
			get_relay(relay,
					  org.omg.Security.DelegationDirective.Delegate);
		relay_no_delegate = 
			get_relay(relay,
					  org.omg.Security.DelegationDirective.NoDelegate);

		SecurityMechanismData[] data = current.get_security_mechanisms(relay_no_delegate);
		our_security_name = orbasec.corba.CredUtil.getAccessId(current.own_credentials()[0]);
		relay_security_name = Opaque.encodeKerberosName(
									new String(data[0].security_name)).getEncoding();
	}
	
	private static void test()
	{
		boolean ret;
		int exitval = 0;
		ret = hello_no_delegate.hello(our_security_name,
			org.omg.Security.DelegationMode.SecDelModeNoDelegation);
		if (!ret) {
			System.out.println("Test Failed on hello_no_delegate.");
			exitval |= 1;
		}
		ret = hello_delegate.hello(our_security_name,
			org.omg.Security.DelegationMode.SecDelModeSimpleDelegation);
		if (!ret) {
			System.out.println("Test Failed on hello_delegate.");
			exitval |= 2;
		}
		ret = relay_no_delegate.relay(relay_security_name,
			org.omg.Security.DelegationMode.SecDelModeNoDelegation);
		if (!ret) {
			System.out.println("Test Failed on relay_no_delegate.");
			exitval |= 4;
		}
		ret = relay_delegate.relay(our_security_name,
			org.omg.Security.DelegationMode.SecDelModeNoDelegation);
		if (!ret) {
			System.out.println("Test Failed on relay_delegate.");
			exitval |= 8;
		}
		if (exitval == 0)
			System.out.println("Test Succeeded.");
		System.exit(exitval);
	}

	private static void dump_creds()
	{
		//
		// Print credentials to screen
		//
		org.omg.SecurityLevel2.Credentials[] credlist = 
			current.own_credentials();
		System.out.println("Own Credentials:");
		for(int i = 0; i < credlist.length; i++ ) {	
			   orbasec.corba.CredUtil.dumpCredentials(credlist[i]);
		}
	}
	
	private static void interactive()
	{
		dump_creds();
		
    	//
    	// Main loop
    	//
    	System.out.println("Enter 'hn' or 'hd' for hello [nodelegate,delegate]");
		System.out.println("      'rn' or 'rd' for relay [nodelegate,delegate]");
		System.out.println("      'x' for exit:");
    	try {
			String input;
			do {
				System.out.print("> ");
				DataInputStream dataIn = new DataInputStream(System.in);
				BufferedReader in =
	    			new BufferedReader(new InputStreamReader(dataIn));
				input = in.readLine();

				boolean ret = false;
				if(input.equals("hn")) {
					ret = hello_no_delegate.hello(our_security_name,
						org.omg.Security.DelegationMode.SecDelModeNoDelegation);
	    		} else
				if(input.equals("hd")) {
					ret = hello_delegate.hello(our_security_name,
						org.omg.Security.DelegationMode.SecDelModeSimpleDelegation);
	    		} else
				if(input.equals("rn")) {
					ret = relay_no_delegate.relay(relay_security_name,
						org.omg.Security.DelegationMode.SecDelModeNoDelegation);
				} else
				if(input.equals("rd")) {
					ret = relay_delegate.relay(our_security_name,
						org.omg.Security.DelegationMode.SecDelModeNoDelegation);
				} else
				if (input.equals("x")) {
				} else {
					System.out.println("Command '"+input+"' not understood.");
					continue;
				}
				if (!input.equals("x")) {
					System.out.println("Returns " + ret + ", test " + (ret ? "succeeded." : "failed."));
				}
			} while(!input.equals("x"));
		} catch(IOException ex) {
			System.err.println("Can't read from `" +
				   ex.getMessage() + "'");
			System.exit(1);
		}

		System.exit(0);
	}
	
    //
    // Standalone program initialization
    //
    public static void main(String args[])
    {
    	try {
			init(args);
			authenticate();
			setup_references();
		 
			if (args.length > 0 && args[0] != null && args[0].equals("-test"))
				test(); // This will exit
				
			
			interactive();
			// exits;
    	} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
    	}
    }
}
