//*****************************************************************************/
// Project: ORBAsec
//
// File:    $Id$
//
// Description:
//    This file is based almost entirely on OOC's naming.Client demo.
//    class.  It is modified here to use Orbasec SL2.
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/
// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1997
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************
 
package naming;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import java.io.*;

import orbasec.corba.Opaque;

public class Client
{
    public static void main(String args[])
    {
	int status;

	try
	{
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init( 
				args, properties );
			ORB orb = orbasec.SL2.orb();

	    //
	    // Run tests
	    //
	    status = run(orb, args);
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
		status = 1;
	}

	System.exit(status);
    }

    static void
    printBinding(Binding b)
    {
	int i;

	for(i = 0 ; i < b.binding_name.length ; i++)
	{
	    if(i > 0)
		System.out.print(" ; ");

	    System.out.print(b.binding_name[i].id);
	}

	switch(b.binding_type.value())
	{
	case BindingType._nobject:
	    System.out.println(" (object)");
	    break;

	case BindingType._ncontext:
	    System.out.println(" (context)");
	    break;
	}
    }

    static int
    run(ORB orb, String args[])
    {
// @@@@@ ORBASEC SL2 -->
		try {
			// initialize SL2

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		org.omg.SecurityLevel2.Current current = 
			org.omg.SecurityLevel2.CurrentHelper.narrow(
	    		orb.resolve_initial_references("SecurityCurrent"));

    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = "Kerberos";
			int	    method    = 0;
			byte	security_name[] =
	//@@@@@ Change this to a vaild service principal name 
					Opaque.encodeKerberosName(
					        "naming_demo_client@MYREALM.COM").getEncoding();
			byte	auth_data[] =
			    	("delegation=false\n" +
					 "config=FILE:orbasec_krb5.conf\n" +
					 "cache_name=MEMORY:0\n" +
					 "password=\"mypassword\"").getBytes();
			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds_holder =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds_holder,
				continuation_data,
				auth_specific_data
			);

			//
			// Print credentials to screen
			//
			org.omg.SecurityLevel2.Credentials[] credlist = 
				current.own_credentials();
			System.out.println("Own Credentials:");
			orbasec.corba.CredUtil.dumpCredentials(credlist);
		} catch ( org.omg.CORBA.ORBPackage.InvalidName invalid_name ) {
			invalid_name.printStackTrace();
			System.exit( 1 );
		}
		
		// Once we are authenticated, add any initial services
		// specified on the command line 
		// (in this case, the naming service)
		orbasec.SL2.add_initial_services();
// @@@@@  <-- ORBASEC SL2


	//
	// Get naming service
	//
	org.omg.CORBA.Object obj = null;

	try
	{
	    obj = orb.resolve_initial_references("NameService");
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    System.err.println("Can't resolve `NameService'");
	    return 1;
	}

	if(obj == null)
	{
	    System.err.println("`NameService' is a nil object reference");
	    return 1;
	}

	NamingContext nc = NamingContextHelper.narrow(obj);

	if(nc == null)
	{
	    System.err.println("`NameService' is not " +
			       "a NamingContext object reference");
	    return 1;
	}

	//
	// Get objects by name
	//
	try
	{
	    int i;

	    //
	    // Resolve names with the Naming Service
	    //
	    NameComponent[] aName = new NameComponent[1];
	    aName[0] = new NameComponent();
	    aName[0].id = "a";
	    aName[0].kind = "";
	    org.omg.CORBA.Object aObj = nc.resolve(aName);
	    Named a = NamedHelper.narrow(aObj);
	    if(a == null)
		throw new RuntimeException();
	    System.out.println("Resolved `a'");

	    NameComponent[] nc1Name = new NameComponent[1];
	    nc1Name[0] = new NameComponent();
	    nc1Name[0].id = "nc1";
	    nc1Name[0].kind = "";
	    org.omg.CORBA.Object nc1Obj = nc.resolve(aName);
	    Named nc1 = NamedHelper.narrow(nc1Obj);
	    if(nc1 == null)
		throw new RuntimeException();
	    System.out.println("Resolved `nc1'");

	    NameComponent[] bName = new NameComponent[2];
	    bName[0] = new NameComponent();
	    bName[0].id = "nc1";
	    bName[0].kind = "";
	    bName[1] = new NameComponent();
	    bName[1].id = "b";
	    bName[1].kind = "";
	    org.omg.CORBA.Object bObj = nc.resolve(bName);
	    Named b = NamedHelper.narrow(bObj);
	    if(b == null)
		throw new RuntimeException();
	    System.out.println("Resolved `b' in naming conext `nc1'");

	    NameComponent[] nc2Name = new NameComponent[2];
	    nc2Name[0] = new NameComponent();
	    nc2Name[0].id = "nc1";
	    nc2Name[0].kind = "";
	    nc2Name[1] = new NameComponent();
	    nc2Name[1].id = "nc2";
	    nc2Name[1].kind = "";
	    org.omg.CORBA.Object nc2Obj = nc.resolve(aName);
	    Named nc2 = NamedHelper.narrow(nc2Obj);
	    if(nc2 == null)
		throw new RuntimeException();
	    System.out.println("Resolved `nc12' in naming context `nc1'");

	    NameComponent[] cName = new NameComponent[3];
	    cName[0] = new NameComponent();
	    cName[0].id = "nc1";
	    cName[0].kind = "";
	    cName[1] = new NameComponent();
	    cName[1].id = "nc2";
	    cName[1].kind = "";
	    cName[2] = new NameComponent();
	    cName[2].id = "c";
	    cName[2].kind = "";
	    org.omg.CORBA.Object cObj = nc.resolve(cName);
	    Named c = NamedHelper.narrow(cObj);
	    if(c == null)
		throw new RuntimeException();
	    System.out.println("Resolved `c' in naming conext `nc2'" +
			       "in naming context  `nc'");

	    System.out.println("Listing all bindings:");
	    BindingListHolder bl = new BindingListHolder();
	    BindingIteratorHolder bi = new BindingIteratorHolder();
	    nc.list(999999, bl, bi);
	    for(i = 0 ; i < bl.value.length ; i++)
		printBinding(bl.value[i]);
	    if(bi == null)
		throw new RuntimeException();

	    System.out.println("Now listing only two bindings:");
	    BindingListHolder bl2 = new BindingListHolder();
	    BindingIteratorHolder bi2 = new BindingIteratorHolder();
	    nc.list(2, bl2, bi2);
	    for(i = 0 ; i < bl2.value.length ; i++)
		printBinding(bl2.value[i]);
	    if(bi2 == null)
		throw new RuntimeException();

	    System.out.println("Listing the rest with the binding iterator:" );
	    BindingListHolder bl3 = new BindingListHolder();
	    boolean bo = bi2.value.next_n(999999, bl3);
	    if(bo != false)
		throw new RuntimeException();
	    for(i = 0 ; i < bl3.value.length ; i++)
		printBinding(bl3.value[i]);
	    bi2.value.destroy();
	}
	catch(NotFound ex)
	{
	    System.err.print("Got a `NotFound' exception (");
	    switch(ex.why.value())
	    {
	    case NotFoundReason._missing_node:
		System.err.print("missing node");
		break;

	    case NotFoundReason._not_context:
		System.err.print("not context");
		break;

	    case NotFoundReason._not_object:
		System.err.print("not object");
		break;
	    }
	    System.err.println(")");
	    ex.printStackTrace();
	    return 1;
	}
	catch(CannotProceed ex)
	{
	    System.err.println("Got a `CannotProceed' exception");
	    ex.printStackTrace();
	    return 1;
	}
	catch(InvalidName ex)
	{
	    System.err.println("Got an `InvalidName' exception");
	    ex.printStackTrace();
	    return 1;
	}

	return 0;
    }
}
