//*****************************************************************************/
// Project: ORBAsec
//
// File:    $Id$
//
// Description:
//    This file is based almost entirely on OOC's com.ooc.CosNaming.Server
//    class.  It is modified here to use Orbasec SL2.
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/
// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNaming;

import org.omg.CORBA.*;
import java.io.*;

import orbasec.corba.Opaque;

final public class NameService
{
    public static void main(String args[])
    {
		int status;
        try
        {
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			ORB orb = orbasec.SL2.orb();

	    //
	    // Filter options
	    //
	    args = ((com.ooc.CORBA.ORB)orb).filter_options(args);

	    status = run(orb, args);
	}
	catch(SystemException ex)
        {
            ex.printStackTrace();
            status = 1;
        }

	System.exit(status);
    }

    static int
    run(ORB orb, String[] args)
    {
        try
        {
			BOA boa = orbasec.SL2.boa();
// @@@@@ ORBASEC SL2 -->
		try {

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		org.omg.SecurityLevel2.Current current = 
			org.omg.SecurityLevel2.CurrentHelper.narrow(
	    		orb.resolve_initial_references("SecurityCurrent"));

    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = "Kerberos";
			int	    method    = 0;
			byte	security_name[] =
	//@@@@@ Change this to a vaild principal name 
					Opaque.encodeKerberosName(
					         "name_server@MYREALM.COM")
							      .getEncoding();

			byte	auth_data[] =
			        (
//@@@@ Change this to principal's password!
				     "password=\"mypassword\"\n" +
//@@@@ Credentials will not have delegation ability.
					 "delegation=false\n" +
//@@@@ Using a memory credentials cache.
				     "cache_name=MEMORY:0\n" +
//@@@@@ Using a memory keytab which allows this
//@@@@@ server to accept security associations.
				     "keytab=MEMORY:0\n" +
				     "enable_server=true\n" +
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();

			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds_holder =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds_holder,
				continuation_data,
				auth_specific_data
			);

			//
			// Print credentials to screen
			//
			org.omg.SecurityLevel2.Credentials[] credlist = 
				current.own_credentials();
			System.err.println("Own Credentials:");
			orbasec.corba.CredUtil.dumpCredentials(System.err,credlist);
		} catch ( org.omg.CORBA.ORBPackage.InvalidName invalid_name ) {
			invalid_name.printStackTrace();
			System.exit( 1 );
		}
// @@@@@  <-- ORBASEC SL2

            String progName = "com.ooc.CosNaming.Server";

            String service = "NameService";
            org.omg.CORBA.Object serviceObject = null;
            try
            {
                serviceObject = orb.resolve_initial_references(service);
            }
            catch(org.omg.CORBA.ORBPackage.InvalidName e)
            {
            }

	    //
	    // Set the IIOP port based on the port defined in the IIOP profile
	    // of the service object (if it exists). Then create the BOA.
	    //
	    // is this valid after BOA has been initted?
		com.ooc.CORBA.BOA.set_iiop_port_from_object(serviceObject);

	    //
	    // Filter options
	    //
	    args = ((com.ooc.CORBA.BOA)boa).filter_options(args);

	    //
	    // Get options
	    //
	    boolean ior = false;
	    boolean start = false;
	    int timeout = 5; // Every 5 minutes.
	    String database = null;

	    //
	    // Check properties first.
	    //
	    com.ooc.CORBA.Properties properties =
		com.ooc.CORBA.Properties.instance();
	    String[] keys = properties.getKeys("ooc.naming.");
	    for(int i = 0 ; i < keys.length ; ++i)
	    {
		String key = keys[i];
		String value = properties.getProperty(key);

		if(key.equals("ooc.naming.database"))
		{
		    database = value;
		}
		else if(key.equals("ooc.naming.timeout"))
		{
		    try
		    {
		        timeout = Integer.parseInt(value);
		    }
		    catch(NumberFormatException ex)
		    {
			System.err.println(
			    "Invalid value for ooc.naming.timeout");
			return 1;
		    }
		}
		else
		{
		    System.err.println(progName + ": `" + key + "': " +
				       "unknown property");
		}
	    }

            for(int i = 0 ; i < args.length && args[i].startsWith("-") ; i++)
            {
                if(args[i].equals("--help") ||
                   args[i].equals("-h"))
                {
                    usage(progName);
                    return 0;
                }
                else if(args[i].equals("--version") ||
                        args[i].equals("-v"))
                {
                    System.err.println("Naming Service " +
                                       com.ooc.CORBA.Version.version);
                    return 0;
                }
                else if(args[i].equals("--ior") ||
                        args[i].equals("-i"))
                {
                    ior = true;
                }
		else if(args[i].equals("--timeout") ||
                        args[i].equals("-t"))
                {
                    ++i;
		    if(i >= args.length)
		    {
			System.err.println(progName + ": " + args[i - 1] +
					   "expects argument");
			usage(progName);
			return 1;
		    }

		    timeout = Integer.parseInt(args[i]);
                }
		else if(args[i].equals("--database") ||
                        args[i].equals("-d"))
                {
		    ++i;
		    if(i >= args.length)
		    {
			System.err.println(progName + ": " + args[i - 1] +
					   "expects argument");
			usage(progName);
			return 1;
		    }

		    database = args[i];
		}
		else if(args[i].equals("--start") ||
                        args[i].equals("-s"))
                {
		    start = true;
		}
		else
		{
		    System.err.println(progName + ": unknown option `"
				       + args[i] + "'\n");
		    usage(progName);
		    return 1;
		}
            }

	    if(timeout <= 0)
	    {
		System.err.println(progName + ": timeout value must be > 0");
		return 1;
	    }

	    if(start && database == null)
	    {
		System.err.println(progName +
				   ": --start (-s) requires the the " +
				   "--database (-d) option");
		return 1;
	    }

	    NamingContextSet ncs = new NamingContextSet();
	    NamingDatabase store = null;

	    org.omg.CosNaming.NamingContext root = null;

	    if(database != null)
	    {
		//
		// Create a new NamingDatabase
		//
		store = new NamingDatabase(boa, orb, database, ncs, start);
 
		//
		// Start logger
		//
		(new Logger(store, timeout * 1000)).start();
	    }
	    else
		System.err.println("Running in non-persistent mode.");

	    if(start || database == null)
	    {
		org.omg.CosNaming.NamingContext nc =
		    new com.ooc.CosNaming.NamingContext(
			orb, store, ncs, "DefaultNamingContext");

		((com.ooc.CORBA.ORB)orb).conc_model(
		    com.ooc.CORBA.ORB.ConcModel.ConcModelThreaded);
		((com.ooc.CORBA.BOA)boa).conc_model(
		    com.ooc.CORBA.BOA.ConcModel.ConcModelThreaded);

		//
		// Print IOR on standard output
		//
		if(ior)
		{
		    String s = orb.object_to_string(nc);
		    System.out.println(s);
		}
	    }

            //
            // Run implementation
            //
            boa.impl_is_ready(null);
 
            return 0;
        }
        catch(SystemException ex)
        {
            ex.printStackTrace();
            return 1;
        }
    }

    static void
    usage(String progName)
    {
        System.err.println("Usage:");
        System.err.println(progName + " [options]\n" +
                           "\n" +
                           "Options:\n" +
                           "-h, --help             " +
                           "Show this message.\n" +
                           "-v, --version          " +
                           "Show NamingService version.\n" +
                           "-i, --ior              " +
                           "Print IOR on standard output.\n" +
			   "-s, --start            " +
			   "Run for the first time.\n" +
			   "-d, --database FILE    " +
			   "Use FILE as the database.\n" +
			   "-t, --timeout MINS     " +
			   "Purge the database every MINS minutes.\n" +
			   "                       " +
                           "(The default is 5 minutes.)\n");
    }
}
