//*****************************************************************************/
// Project: ORBAsec
//
// File:    $Id$
//
// Description:
//    This file is based almost entirely on OOC's naming.Server demo.
//    class.  It is modified here to use Orbasec SL2.
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/
// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************
 
package naming;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import java.io.*;

import orbasec.corba.Opaque;

public class Server
{
    public static void main(String args[])
    {
	int status;
	try
	{
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			ORB orb = orbasec.SL2.orb();

	    status = run(orb, args);
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	System.exit(status);
    }

    static int
    run(ORB orb, String[] args)
    {
	String namingFile = null;

	//
	// Parse command line arguments
	//
	int argc = 0;
	while(argc < args.length)
	{
	    if(args[argc].equals("-f"))
	    {
                if(argc + 1 < args.length)
                {
                    argc++;
                    namingFile = args[argc];
                }
                else
                    usage();
	    }
	    else if(args[argc].equals("-h") || args[argc].equals("--help"))
		usage();
 
            argc++;
	}

			BOA boa = orbasec.SL2.boa();

// @@@@@ ORBASEC SL2 -->
		try {

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		org.omg.SecurityLevel2.Current current = 
			org.omg.SecurityLevel2.CurrentHelper.narrow(
	    		orb.resolve_initial_references("SecurityCurrent"));

    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = "Kerberos";
			int	    method    = 0;
			byte	security_name[] =
	//@@@@@ Change this to a vaild service principal name 
					Opaque.encodeKerberosName(
                        "marge@MYREALM.COM"
					).getEncoding();

			byte	auth_data[] =
			        (
//@@@@ Change this to principal's password!
				     "password=\"mypassword\"\n" +
//@@@@ Credentials will not have delegation ability.
					 "delegation=false\n" +
//@@@@ Using a memory credentials cache.
				     "cache_name=MEMORY:0\n" +
//@@@@@ Using a memory keytab which allows this
//@@@@@ server to accept security associations.
				     "keytab=MEMORY:0\n" +
				     "enable_server=true\n" +
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();

			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds_holder =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds_holder,
				continuation_data,
				auth_specific_data
			);

			//
			// Print credentials to screen
			//
			org.omg.SecurityLevel2.Credentials[] credlist = 
				current.own_credentials();
			System.out.println("Own Credentials:");
			orbasec.corba.CredUtil.dumpCredentials(credlist);
		} catch ( org.omg.CORBA.ORBPackage.InvalidName invalid_name ) {
			invalid_name.printStackTrace();
			System.exit( 1 );
		}
		
		// Once we are authenticated, add any initial services
		// specified on the command line 
		// (in this case, the naming service)
		orbasec.SL2.add_initial_services();
// @@@@@  <-- ORBASEC SL2


	//
	// Create some implementations
	//
	Named a = new Named_impl();
	Named a1 = new Named_impl();
	Named a2 = new Named_impl();
	Named a3 = new Named_impl();
	Named b = new Named_impl();
	Named c = new Named_impl();

	//
	// Get naming servie
	//
	org.omg.CORBA.Object obj = null;

	if(namingFile != null)
	{
	    try
	    {
		FileReader r = new FileReader(namingFile);
		BufferedReader in = new BufferedReader(r);
		String ref = in.readLine();
		r.close();
 
		obj = orb.string_to_object(ref);
	    }
	    catch(IOException e)
	    {
		e.printStackTrace();
		System.exit(1);
	    }
	}
	else
	{
	    try
	    {
		obj = orb.resolve_initial_references("NameService");
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.out.println("Can't resolve `NameService'");
		System.exit(1);
	    }
	}

	if(obj == null)
	{
	    System.out.println("`NameService' is a nil object reference");
	    System.exit(1);
	}

	NamingContext nc =
	    NamingContextHelper.narrow(obj);

	if(nc == null)
	{
	    System.out.println("`NameService' is not " +
			       "a NamingContext object reference");
	    System.exit(1);
	}

	try
	{
	    //
	    // Create and bind some Naming Contexts
	    //
	    NameComponent[] nc1Name = new NameComponent[1];
	    nc1Name[0] = new NameComponent();
	    nc1Name[0].id = "nc1";
	    nc1Name[0].kind = "";
	    NamingContext nc1 = nc.bind_new_context(nc1Name);

	    NameComponent[] nc2Name = new NameComponent[2];
	    nc2Name[0] = new NameComponent();
	    nc2Name[0].id = "nc1";
	    nc2Name[0].kind = "";
	    nc2Name[1] = new NameComponent();
	    nc2Name[1].id = "nc2";
	    nc2Name[1].kind = "";
	    NamingContext nc2 = nc.bind_new_context(nc2Name);

	    //
	    // Bind names with the Naming Service
	    //
	    NameComponent[] aName = new NameComponent[1];
	    aName[0] = new NameComponent();
	    aName[0].id = "a";
	    aName[0].kind = "";
	    nc.bind(aName, a);

	    NameComponent[] a1Name = new NameComponent[1];
	    a1Name[0] = new NameComponent();
	    a1Name[0].id = "a1";
	    a1Name[0].kind = "";
	    nc.bind(a1Name, a1);

	    NameComponent[] a2Name = new NameComponent[1];
	    a2Name[0] = new NameComponent();
	    a2Name[0].id = "a2";
	    a2Name[0].kind = "";
	    nc.bind(a2Name, a2);

	    NameComponent[] a3Name = new NameComponent[1];
	    a3Name[0] = new NameComponent();
	    a3Name[0].id = "a3";
	    a3Name[0].kind = "";
	    nc.bind(a3Name, a3);

	    NameComponent[] bName = new NameComponent[2];
	    bName[0] = new NameComponent();
	    bName[0].id = "nc1";
	    bName[0].kind = "";
	    bName[1] = new NameComponent();
	    bName[1].id = "b";
	    bName[1].kind = "";
	    nc.bind(bName, b);

	    NameComponent[] cName = new NameComponent[3];
	    cName[0] = new NameComponent();
	    cName[0].id = "nc1";
	    cName[0].kind = "";
	    cName[1] = new NameComponent();
	    cName[1].id = "nc2";
	    cName[1].kind = "";
	    cName[2] = new NameComponent();
	    cName[2].id = "c";
	    cName[2].kind = "";
	    nc.bind(cName, c);

	    //
	    // Run implementation
	    //
	    boa.impl_is_ready(null);

	    //
	    // Unregister names with the Naming Service
	    //
	    nc.unbind(cName);
	    nc.unbind(bName);
	    nc.unbind(aName);
	    nc.unbind(nc2Name);
	    nc.unbind(nc1Name);
	}
	catch(NotFound ex)
	{
	    System.err.print("Got a `NotFound' exception (");
	    switch(ex.why.value())
	    {
	    case NotFoundReason._missing_node:
		System.err.print("missing node");
		break;

	    case NotFoundReason._not_context:
		System.err.print("not context");
		break;

	    case NotFoundReason._not_object:
		System.err.print("not object");
		break;
	    }
	    System.err.println(")");
	    ex.printStackTrace();
	    return 1;
	}
	catch(CannotProceed ex)
	{
	    System.err.println("Got a `CannotProceed' exception");
	    ex.printStackTrace();
	    return 1;
	}
	catch(InvalidName ex)
	{
	    System.err.println("Got an `InvalidName' exception");
	    ex.printStackTrace();
	    return 1;
	}
	catch(AlreadyBound ex)
	{
	    System.err.println("Got an `AlreadyBound' exception");
	    ex.printStackTrace();
	    return 1;
	}

	return 0;
    }

    private static void
    usage()
    {
	System.out.println(
	    "Usage: java naming.Server [options]\n\n" +
	    "Options:\n" +
	    "-f FILE     Read the NamingService IOR from file FILE.\n" +
	    "-h, --help  Display this help message.");

        System.exit(1);
    }
}
 
