//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/

package hello;

import org.omg.CORBA.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import orbasec.corba.*;
import orbasec.corba.Opaque;
import org.omg.Security.*;

public class Client
{
	static ORB orb;
	static BOA boa;
	static orbasec.SecLev2.Current current;

	
	// Object reference with different DelegationDirective Policies on them
	static Hello  hello_no_prot, hello_integ, hello_conf, hello_integ_conf;

	// Intended Security Names for the Client and the Relay that
	// the Hello server should verify;
	static org.omg.Security.SecAttribute our_accessid;
	
	private static Hello get_hello()
	{
    	// 
		// Get "hello" object
    	//
		System.out.println("Getting Hello Reference");
    	String ref = null;
    	try {
			String refFile = "Hello.ref";
			FileInputStream file = new FileInputStream(refFile);
			BufferedReader in =
				new BufferedReader(new InputStreamReader(file));
			ref = in.readLine();
			file.close();
    	} catch(IOException ex) {
			System.err.println("Can't read from `" +
				   ex.getMessage() + "'");
			System.exit(1);
    	}

    	org.omg.CORBA.Object obj = orb.string_to_object(ref);
    	if(obj == null)
			throw new RuntimeException();


		Hello hello = HelloHelper.narrow(orb.string_to_object(ref));
		if(hello == null)
			throw new RuntimeException();
		return hello;
	}
	
	private static Hello 
	get_hello(Hello hello, 
		org.omg.Security.QOP      qop
	)
		throws org.omg.CORBA.PolicyManagerPackage.InvalidPolicies,
			   org.omg.CORBA.ORBPackage.InvalidName
	{
		//
		// Modify Object Invocation Policies
		//
		org.omg.CORBA.Policy[] policies;
		policies = new org.omg.CORBA.Policy[1];

		policies[0] = 
		orbasec.SL2.create_qop_policy(qop);
		
		//
		// Set the policy on the Object reference
		//
		return
			HelloHelper.narrow(
				hello._set_policy_overrides(
					policies, 
					org.omg.CORBA.SetOverrideType.ADD_OVERRIDE));
	}
	
	private static void init(String args[])
		throws org.omg.CORBA.ORBPackage.InvalidName
	{
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			orb = orbasec.SL2.orb();
			boa = orbasec.SL2.boa();

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		current = 
				orbasec.SecLev2.CurrentHelper.narrow(
					orb.resolve_initial_references("SecurityCurrent"));
	}

	private static void authenticate()
	{
    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = "Kerberos";
			int	    method    = 0;
			byte	security_name[] = 
//@@@@ Change this to a valid kerberos principal!
				    Opaque.encodeKerberosName(
					     "hello_demo_client@MYREALM.COM"
					).getEncoding();

			byte	auth_data[] =
			        (
//@@@@ Change this to principal password!
				     "password=\"mypassword\"\n" +
//@@@@ Setting delegation capability off	
					 "delegation=false\n"+
//@@@@ Using a memory credentials cache
				     "cache_name=MEMORY:0\n"+
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();

			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds,
				continuation_data,
				auth_specific_data
			);

	}
	
	private static void setup_references()
		throws  org.omg.CORBA.PolicyManagerPackage.InvalidPolicies,
		        org.omg.CORBA.ORBPackage.InvalidName
	{
		Hello hello = get_hello();

		hello_no_prot = 
			get_hello(hello,org.omg.Security.QOP.SecQOPNoProtection);
			
		hello_integ = 
			get_hello(hello,org.omg.Security.QOP.SecQOPIntegrity);

		hello_conf = 
			get_hello(hello,org.omg.Security.QOP.SecQOPConfidentiality);

		hello_integ_conf = 
			get_hello(hello,org.omg.Security.QOP.SecQOPIntegrityAndConfidentiality);

		our_accessid = orbasec.corba.CredUtil.getAccessIdAttribute(
								current.own_credentials()[0]);
	}
	
	private static boolean
	has(short options, short qop)
	{
		return qop == (options & mask);
	}
	
	static short noprot    = org.omg.Security.NoProtection.value,
		integ     = org.omg.Security.Integrity.value,
		conf      = org.omg.Security.Confidentiality.value,
		integconf = (short)(integ | conf),
		mask      = (short) (noprot | integ | conf);
		
	private static void test()
	{
		boolean ret;
		int     exitval = 0;
		org.omg.Security.SecAttributeHolder accessid = 
			new org.omg.Security.SecAttributeHolder();
		org.omg.CORBA.ShortHolder opts = 
			new org.omg.CORBA.ShortHolder();
		
		hello_no_prot.hello(accessid,opts);
		if (!orbasec.corba.CredUtil.equals(our_accessid,accessid.value) ||
			!has(opts.value,noprot)) {
			System.out.println("Test Failed on hello_no_prot.");
			exitval |= 1;
		}
		hello_integ.hello(accessid,opts);
		if (!orbasec.corba.CredUtil.equals(our_accessid,accessid.value) ||
			!has(opts.value,integ)) {
			System.out.println("Test Failed on hello_integ.");
			exitval |= 1;
		}
		
		// Confidentiality Only doesn't work, and shouldn't in
		// most cases, so it should arrive at the server with
		// I & C.
		hello_conf.hello(accessid,opts);
		if (!orbasec.corba.CredUtil.equals(our_accessid,accessid.value) ||
			!has(opts.value,integconf)) {
			System.out.println("Test Failed on hello_conf.");
			exitval |= 1;
		}
		hello_integ_conf.hello(accessid,opts);
		if (!orbasec.corba.CredUtil.equals(our_accessid,accessid.value) ||
			!has(opts.value,integconf)) {
			System.out.println("Test Failed on hello_integ_conf.");
			exitval |= 1;
		}
		if (exitval == 0)
			System.out.println("Test Succeeded.");
		System.exit(exitval);
	}

	private static void dump_creds()
	{
		//
		// Print credentials to screen
		//
		org.omg.SecurityLevel2.Credentials[] credlist = 
			current.own_credentials();
		System.out.println("Own Credentials:");
		orbasec.corba.CredUtil.dumpCredentials(credlist);
	}
	
	
    //
    // Standalone program initialization
    //
    public static void main(String args[])
    {
    	try {
			init(args);
			authenticate();
			dump_creds();
			setup_references();
			
			test(); // This will exit
				
   	} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
    	}
    }
}
