//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/

package hello;

import org.omg.CORBA.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import orbasec.corba.*;
import orbasec.corba.Opaque;
import org.omg.Security.*;

public class ClientKRB
{
	static ORB orb;
	static BOA boa;
	static orbasec.SecLev2.Current current;

	
	// Object reference 
	static Hello  hello1, hello2, hello3;

	// Intended Security Names for the Client and the Relay that
	// the Hello server should verify;
	static org.omg.Security.SecAttribute our_accessid;
	
	// The string form of the our_accessid.value
	static String                        our_name;
	
	private static Hello get_hello( String refFile )
	{
    	// 
		// Get "hello" object
    	//
    	String ref = null;
    	try {
			FileInputStream file = new FileInputStream(refFile);
			BufferedReader in =
				new BufferedReader(new InputStreamReader(file));
			ref = in.readLine();
			file.close();
    	} catch(IOException ex) {
			System.err.println("Can't read from `" +
				   ex.getMessage() + "'");
			System.exit(1);
    	}

    	org.omg.CORBA.Object obj = orb.string_to_object(ref);
    	if(obj == null)
			throw new RuntimeException();


		Hello hello = HelloHelper.narrow(orb.string_to_object(ref));
		if(hello == null)
			throw new RuntimeException();
		return hello;
	}
	
	private static void init(String args[])
		throws org.omg.CORBA.ORBPackage.InvalidName
	{
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			orb = orbasec.SL2.orb();
			boa = orbasec.SL2.boa();

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		current = 
				orbasec.SecLev2.CurrentHelper.narrow(
					orb.resolve_initial_references("SecurityCurrent"));
	}

	private static void authenticate()
	{
    	org.omg.SecurityLevel2.PrincipalAuthenticator pa = 
			current.principal_authenticator();

		String  mechanism = "Kerberos";
		int	    method    = 0;

		byte	security_name[] =
				orbasec.corba.Opaque.encodeKerberosName(
					"marge@MYREALM.COM"
				).getEncoding();

		byte	auth_data[] =
			        (
//@@@@ Change this to principal password!
				     "password=\"mypassword\"\n" +
//@@@@ Setting delegation capability off	
					 "delegation=false\n"+
//@@@@ Using a memory credentials cache
				     "cache_name=MEMORY:0\n"+
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();

		org.omg.Security.SecAttribute privileges[] =
			new org.omg.Security.SecAttribute[0];
		org.omg.SecurityLevel2.CredentialsHolder creds_holder =
			new org.omg.SecurityLevel2.CredentialsHolder();
		org.omg.Security.OpaqueHolder
		 continuation_data =
			new org.omg.Security.OpaqueHolder(),
		 auth_specific_data =
			new org.omg.Security.OpaqueHolder();

		pa.authenticate(
			method,
			mechanism,
			security_name,
			auth_data,
			privileges,
			creds_holder,
			continuation_data,
			auth_specific_data
		);
	}
	
	private static void setup_references()
		throws  org.omg.CORBA.PolicyManagerPackage.InvalidPolicies,
		        org.omg.CORBA.ORBPackage.InvalidName
	{
		try {
			hello1 = get_hello("Hello-krbssl.ref");
		} catch (org.omg.CORBA.SystemException e) {
			System.err.println("Can't get to Hello-krbssl.ref");
			System.err.println("Test Failed.");
			System.exit(1);
		}
		System.err.println("Got Hello-krbssl: " + hello1);
		try {
			hello2 = get_hello("Hello-ssl.ref");
		} catch (org.omg.CORBA.SystemException e) {
			System.err.println("Can't get to Hello-krbssl.ref");
			System.err.println("Test Failed.");
			System.exit(1);
		}
		System.err.println("Got Hello-ssl: " + hello2);
		try {
			hello3 = get_hello("Hello-krb.ref");
		} catch (org.omg.CORBA.SystemException e) {
			System.err.println("Can't get Hello-krb.ref");
			System.err.println("Test Failed.");
			System.exit(1);
		}
		System.err.println("Got Hello-krb: " + hello3);
		our_accessid = orbasec.corba.CredUtil.getAccessIdAttribute(
								current.own_credentials()[0]);
		try {
			our_name = Opaque.decode(our_accessid.value).toString();
		} catch (Opaque.CodingException e) {
			our_name = "<error decoding>";
		}
		System.err.println("Getting references succeded.");
	}
	
	private static void test()
	{
		boolean ret;
		int     exitval = 0;
		org.omg.Security.SecAttributeHolder accessid = 
			new org.omg.Security.SecAttributeHolder();
		org.omg.CORBA.ShortHolder opts = 
			new org.omg.CORBA.ShortHolder();
		
		hello1.hello(accessid);
		if (!orbasec.corba.CredUtil.equals(our_accessid,accessid.value)) {
			System.out.println("Test Failed on hello-krbssl for \"" + our_name + "\".");
			System.exit( 1 );
		}
		
		try {
			hello2.hello(accessid);
			System.err.println( "ERROR: Accessed hello-ssl, but we have no SSL Credentials!" );
			System.exit( 1 );
		} catch ( org.omg.CORBA.SystemException se ) {
			System.out.println( "Test succeeded on hello-ssl for \"" + our_name + "\"." );
		}
		
		hello3.hello(accessid);
		if (!orbasec.corba.CredUtil.equals(our_accessid,accessid.value)) {
			System.out.println("Test Failed on hello-krb for \"" + our_name + "\".");
			System.exit( 1 );
		}


		System.out.println("Test Succeeded for \"" + our_name + "\".");
		System.exit(0);
	}


	private static void dump_creds()
	{
		//
		// Print credentials to screen
		//
		org.omg.SecurityLevel2.Credentials[] credlist = 
			current.own_credentials();
		System.out.println("Own Credentials:");
		orbasec.corba.CredUtil.dumpCredentials(credlist);
	}
	
	
    //
    // Standalone program initialization
    //
    public static void main(String args[])
    {
    	try {
			init(args);
			authenticate();
			dump_creds();
			setup_references();
			
			test(); // This will exit
				
   	} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
    	}
    }
}
