//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// File:    Server.java
// Date:    August 1998
//
// Description:
//              This file is a modification of ORBacus 3.0 hello demo modified
//   to illustrate Adiron's ORBAsec SL2 (SecurityLevel2).  
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/

package hello;

import org.omg.CORBA.*;
import java.io.*;
import orbasec.corba.Opaque;

public class Server
{
	static org.omg.CORBA.ORB                   orb = null;
	static org.omg.CORBA.BOA                   boa = null;
	static orbasec.SecLev2.Current         current = null;
	
	static org.omg.SecurityLevel2.Credentials[] all_creds = 
		new org.omg.SecurityLevel2.Credentials[2];
	static org.omg.SecurityLevel2.Credentials[] krb_creds = 
		new org.omg.SecurityLevel2.Credentials[1];
	static org.omg.SecurityLevel2.Credentials[] ssl_creds = 
		new org.omg.SecurityLevel2.Credentials[1];
	
	static void
	init(String args[])
	{
		//
		// ORB, BOA, and SL2 initialization
		//
		java.util.Properties properties = new java.util.Properties();
		orbasec.SL2.init_with_boa( 
			args, properties, properties );
		orb = orbasec.SL2.orb();
		boa = orbasec.SL2.boa();
		
		try {
			current = 
				orbasec.SecLev2.CurrentHelper.narrow(
	    			orb.resolve_initial_references("SecurityCurrent"));
		} catch ( org.omg.CORBA.ORBPackage.InvalidName invalid_name ){
			throw new RuntimeException( invalid_name.toString() );
		}
	}
	
	static void
	authenticate_kerberos_creds()
	{
    	org.omg.SecurityLevel2.PrincipalAuthenticator pa = 
			current.principal_authenticator();

		String  mechanism = "Kerberos";
		int	    method    = 0;
			
		byte	security_name[] =
				    Opaque.encodeKerberosName(
//@@@@@ Change this to a vaild kerberos principal name 
				         "homer@MYREALM.COM"
					).getEncoding();
					
		byte	auth_data[] =
			        (
//@@@@ Change this to principal's password!
				     "password=\"mypassword\"\n" +
//@@@@ Credentials will not have delegation ability.
					 "delegation=false\n" +
//@@@@ Using a memory credentials cache.
				     "cache_name=MEMORY:0\n" +
//@@@@@ Using a memory keytab which allows this
//@@@@@ server to accept security associations.
				     "keytab=MEMORY:0\n" +
					 "enable_server=true\n" +
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();
					 
		org.omg.Security.SecAttribute privileges[] =
			new org.omg.Security.SecAttribute[0];
		org.omg.SecurityLevel2.CredentialsHolder creds_holder =
			new org.omg.SecurityLevel2.CredentialsHolder();
		org.omg.Security.OpaqueHolder
		 continuation_data =
			new org.omg.Security.OpaqueHolder(),
		 auth_specific_data =
			new org.omg.Security.OpaqueHolder();

		pa.authenticate(
			method,
			mechanism,
			security_name,
			auth_data,
			privileges,
			creds_holder,
			continuation_data,
			auth_specific_data
		);
		
		all_creds[0] = creds_holder.value;
		krb_creds[0] = creds_holder.value;
	}
	
	static void
	authenticate_ssl_server_creds()
	{
    	org.omg.SecurityLevel2.PrincipalAuthenticator pa = 
			current.principal_authenticator();

		// use exportable non-anonymous credentials
		String  mechanism = orbasec.corba.MechUtil.SSL_DH_DSS_EXPORT_MECH;
		int	    method    = 0;
		byte	security_name[] = new byte[0];
		byte	auth_data[] =
			("cert_file=server.pem\n" +
			 "passphrase=\"blahblah\"\n" +
			 "enable_server=true\n" ).getBytes();
		org.omg.Security.SecAttribute privileges[] =
			new org.omg.Security.SecAttribute[0];
		org.omg.SecurityLevel2.CredentialsHolder creds_holder =
			new org.omg.SecurityLevel2.CredentialsHolder();
		org.omg.Security.OpaqueHolder
		 continuation_data =
			new org.omg.Security.OpaqueHolder(),
		 auth_specific_data =
			new org.omg.Security.OpaqueHolder();

		pa.authenticate(
			method,
			mechanism,
			security_name,
			auth_data,
			privileges,
			creds_holder,
			continuation_data,
			auth_specific_data
		);
		
		// Turn on EstablishTrustInClient requirement
		creds_holder.value.accepting_options_required( (short) 
			(creds_holder.value.accepting_options_required() |
			 org.omg.Security.EstablishTrustInClient.value));

		all_creds[1] = creds_holder.value;
		ssl_creds[0] = creds_holder.value;
	}
	
	static void
	save_reference( String filename, org.omg.CORBA.Object object )
	{
		try {
			String ref = orb.object_to_string( object );
			String refFile = filename;
			FileOutputStream file = new FileOutputStream(refFile);
			PrintWriter out = new PrintWriter(file);
			out.println(ref);
			out.flush();
			file.close();
		} catch(IOException ex) {
			System.err.println("Can't write to `" +
					   ex.getMessage() + "'");
			System.exit(1);
		}
	}

	static boolean
	equiv_lists( 
		org.omg.SecurityLevel2.Credentials[] creds1,
		org.omg.SecurityLevel2.Credentials[] creds2 )
	{
		if ( creds1.length != creds2.length ){
			return false;
		}
		for ( int i = 0;  i < creds1.length;  ++i ){
			boolean occurs = false;
			for ( int j = 0;  j < creds2.length;  ++j ){
				if ( creds2[j] == creds1[i] ){
					occurs = true;
					break;
				}
			}
			if ( !occurs ){
				return false;
			}
		}
		
		return true;
	}

	
	public static void main(String args[])
	{
		try {
			init( args );
			
			// get 3 sets of SSL credentials
			authenticate_kerberos_creds();
			authenticate_ssl_server_creds();
			
			// We will create 3 objects all together.  
			Hello_impl
				hello1,
				hello2,
				hello3;
			
			//-------------------------------------------
			// First, we'll create one before setting our
			// (thread-specific) accepting options.
			hello1     = new Hello_impl();
			hello1.orb = orb;
			save_reference( "Hello-krbssl.ref", hello1 );

			//-------------------------------------------
			// Next, we'll set our accepting options (on this
			// thread) to just the SSL creds.  Check the IOR
			// to verify that indeed the creds are correct.
			current.accepting_credentials( ssl_creds );
			
			hello2     = new Hello_impl();
			hello2.orb = orb;
			save_reference( "Hello-ssl.ref", hello2 );

			//-------------------------------------------
			// Finally, we'll create a our third reference,
			// but we'll set the accepting credentials for
			// this particular reference via the set_credentials()
			// method to be Kerberos-only credentials
			hello3     = new Hello_impl();
			hello3.orb = orb;
			
			// check to see if a BAD_PARAM
			// exception is thrown if we query for creds on
			// an unconnected servant
			try {
				current.get_accepting_credentials( hello3 );
				System.out.println( "Error: Was able to get accepting creds on an unconnected object" );
				System.exit( 1 );
			} catch ( org.omg.CORBA.BAD_PARAM bp1 ){
				System.out.print( "." );
			}
			
			// check to see if a BAD_PARAM
			// exception is thrown if we query for creds on
			// a locality constrained object
			try {
				current.get_accepting_credentials( current );
				System.out.println( "Error: Was able to get accepting creds on a locality-constrained object" );
				System.exit( 1 );
			} catch ( org.omg.CORBA.BAD_PARAM bp1 ){
				System.out.print( "." );
			}
			try {
				current.set_accepting_credentials( current, ssl_creds );
				System.out.println( "Error: Was able to set accepting creds on a locality-constrained object" );
				System.exit( 1 );
			} catch ( org.omg.CORBA.BAD_PARAM bp1 ){
				System.out.print( "." );
			}
			
			// hello3 should only accept krb requests
			current.set_accepting_credentials( hello3, krb_creds );
			save_reference( "Hello-krb.ref", hello3 );
			
			// check to see if a BAD_PARAM
			// exception is thrown if we set the creds
			// on an already connected object
			try {
				current.set_accepting_credentials( hello1, ssl_creds );
				System.out.println( "Error: Was able to set accepting creds on a connected object" );
				System.exit( 1 );
			} catch ( org.omg.CORBA.BAD_PARAM bp1 ){
				System.out.print( "." );
			}
			try {
				current.set_accepting_credentials( hello2, ssl_creds );
				System.out.println( "Error: Was able to set accepting creds on a connected object" );
				System.exit( 1 );
			} catch ( org.omg.CORBA.BAD_PARAM bp1 ){
				System.out.print( "." );
			}
			try {
				current.set_accepting_credentials( hello3, ssl_creds );
				System.out.println( "Error: Was able to set accepting creds on a connected object" );
				System.exit( 1 );
			} catch ( org.omg.CORBA.BAD_PARAM bp1 ){
				System.out.print( "." );
			}
			
			// now check the accepting creds for our 3 servants
			org.omg.SecurityLevel2.Credentials[] credlist;
			
			credlist = current.get_accepting_credentials( hello1 );
			if ( !equiv_lists( credlist, all_creds ) ){
				System.out.println( "Error: hello1 has the wrong accepting credentials" );
				System.exit( 1 );
			} else {
				System.out.print( "." );
			}
			
			credlist = current.get_accepting_credentials( hello2 );
			if ( !equiv_lists( credlist, ssl_creds ) ){
				System.out.println( "Error: hello2 has the wrong accepting credentials" );
				System.exit( 1 );
			} else {
				System.out.print( "." );
			}
			
			credlist = current.get_accepting_credentials( hello3 );
			if ( !equiv_lists( credlist, krb_creds ) ){
				System.out.println( "Error: hello3 has the wrong accepting credentials" );
				System.exit( 1 );
			} else {
				System.out.print( "." );
			}
			
			System.out.println( "\n\n3 servants: " );
			System.out.println( "hello1=" + hello1 );
			System.out.println( "hello2=" + hello2 );
			System.out.println( "hello3=" + hello3 );
			System.out.println( "Server initialization suceeded.  Run Clients." );
			//
			// Run implementation
			//
			System.out.println( "ready." );
			boa.impl_is_ready(null);
			
			System.exit(0);
		} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
		}
	}
}
