//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/

package hello;

import org.omg.CORBA.*;
import java.io.*;
import orbasec.corba.Opaque;

public class Server
{
	// Lets get the authority of our certificate.
	static void
	setupTrustedAuthority(org.omg.SecurityLevel2.Current current,
	                      Opaque security_name)
	{
		try {
			String id = security_name.toString();
			String authority_name = "krbtgt/";

			authority_name += id.substring(id.lastIndexOf('@')+1);
			authority_name += "@";
			authority_name += id.substring(id.lastIndexOf('@')+1);
			
			//authority_name = "krbtgt/MYREALM.COM@MYREALM.COM";
			
			byte[] encoded_name = 
				Opaque.encodeKerberosName(authority_name)
							.getEncoding();
			
			orbasec.SecLev2.TrustedAuthority[] auths = 
				new orbasec.SecLev2.TrustedAuthority[1];
				
			// Length means nothing for Kerberos right now.
			auths[0] = new orbasec.SecLev2.TrustedAuthority(
								"Kerberos",encoded_name, 1);
			
			// We will use this Trusted Authority for all our
			// clients and servers that we connect to, as well as
			// our verification of own principals.
			orbasec.SecLev2.TrustedAuthorityPolicyContent authpol = 
				new orbasec.SecLev2.TrustedAuthorityPolicyContent(
					auths, auths, auths);
					
			orbasec.SecLev2.TrustedAuthorityPolicy p =
				orbasec.SL2.create_trusted_authority_policy(authpol);

			org.omg.CORBA.Policy[] policies = 
				new org.omg.CORBA.Policy[1];

			policies[0] = p;

			((orbasec.SecLev2.Current)current)
				.set_overrides(policies, org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
		} catch (Exception e) {
			e.printStackTrace();
			throw new org.omg.CORBA.INITIALIZE(e.toString());
		}
	}
	public static void main(String args[])
	{
		try {
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			ORB orb = orbasec.SL2.orb();
			BOA boa = orbasec.SL2.boa();

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		org.omg.SecurityLevel2.Current current = 
			org.omg.SecurityLevel2.CurrentHelper.narrow(
	    		orb.resolve_initial_references("SecurityCurrent"));

    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();
			String mechanism = "Kerberos";
			int    method    = 0;
			
//@@@@ Change this to a valid security name.
			Opaque  security_name = 
						Opaque.encodeKerberosName(
									"bart@MYREALM.COM");
									
			byte[]  security_name_param = 
							security_name.getEncoding();						
					
			byte	auth_data[] =
			        (
//@@@@ Change this to principal's password!
				     "password=\"mypassword\"\n" +
//@@@@ Credentials will not have delegation ability.
					 "delegation=false\n" +
//@@@@ Using a memory credentials cache.
				     "cache_name=MEMORY:0\n" +
//@@@@@ Using a memory keytab which allows this
//@@@@@ server to accept security associations.
				     "keytab=MEMORY:0\n" +
				     "enable_server=true\n" +
//@@@@ Kerberos configuration file
					 "config=FILE:orbasec_krb5.conf\n"
					 ).getBytes();
					 
			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds_holder =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			setupTrustedAuthority(current,security_name);
			
			pa.authenticate(
				method,
				mechanism,
				security_name_param,
				auth_data,
				privileges,
				creds_holder,
				continuation_data,
				auth_specific_data
			);

			//
			// Print credentials to screen
			//
			org.omg.SecurityLevel2.Credentials[] credlist = 
				current.own_credentials();
			System.out.println("Own Credentials:");
			orbasec.corba.CredUtil.dumpCredentials(credlist);

			//
			// Create implementation object
			//
			Hello_impl p = new Hello_impl();
			p.orb        = orb;

			//
			// Save reference
			//
			try {
				String ref = orb.object_to_string(p);
				String refFile = "Hello.ref";
				FileOutputStream file = new FileOutputStream(refFile);
				PrintWriter out = new PrintWriter(file);
				out.println(ref);
				out.flush();
				file.close();
			} catch(IOException ex) {
				System.err.println("Can't write to `" +
						   ex.getMessage() + "'");
				System.exit(1);
			}

			//
			// Run implementation
			//
			boa.impl_is_ready(null);

			System.exit(0);
		} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
		}
	}
}
