//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// File:    Client.java
// Date:    August 1998
//
// Description:
//              This file is a modification of ORBacus 3.0 hello demo modified
//   to illustrate Adiron's ORBAsec SL2 (SecurityLevel2).  
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/
// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

package hello;

import org.omg.CORBA.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import orbasec.corba.Opaque;

public class Client
{
    //
    // Standalone program initialization
    //
    public static void main(String args[])
    {
    	try {
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			ORB orb = orbasec.SL2.orb();
			BOA boa = orbasec.SL2.boa();

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		org.omg.SecurityLevel2.Current current = 
			org.omg.SecurityLevel2.CurrentHelper.narrow(
	    		orb.resolve_initial_references("SecurityCurrent"));

    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = orbasec.corba.MechUtil.SSL_DH_DSS_EXPORT_MECH;;
			int	    method    = 0;
			byte	security_name[] = new byte[0]; // assigned from cert
			byte	auth_data[] =
			        ("cert_file=client.pem\n" +
					 "passphrase=\"blahblah\"\n" +
					 "enable_server=false\n" ).getBytes();
			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds,
				continuation_data,
				auth_specific_data
			);

			//
			// Print credentials to screen
			//
			org.omg.SecurityLevel2.Credentials[] credlist = 
				current.own_credentials();
			System.out.println("Own Credentials:");
			for(int i = 0; i < credlist.length; i++ ) {	
				orbasec.corba.CredUtil.dumpCredentials(credlist[i]);
			}

			// Uncomment to experiment with Current Invocation Policies
/* *
			//
			// Modify Invocation Policies
			//
			org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];

			// QOP Policy: use Integrity only
			policies[0] = 
			current.create_qop_policy(org.omg.Security.QOP.SecQOPIntegrity);

			// EstablishTrust Policy: trust in client and target
			policies[1] = current.create_establish_trust_policy(true,true);

			//
			// Set the policies as defaults on Current.
			//
			current.add_policy_overrides(policies);
/* */



    		// 
			// Get "hello" object
    		//
			System.out.println("Getting Hello Reference");
    		String ref = null;
    		try {
				String refFile = "Hello.ref";
				FileInputStream file = new FileInputStream(refFile);
				BufferedReader in =
					new BufferedReader(new InputStreamReader(file));
				ref = in.readLine();
				file.close();
    		} catch(IOException ex) {
				System.err.println("Can't read from `" +
					   ex.getMessage() + "'");
				System.exit(1);
    		}

    		org.omg.CORBA.Object obj = orb.string_to_object(ref);
    		if(obj == null)
				throw new RuntimeException();


			Hello hello = HelloHelper.narrow(orb.string_to_object(ref));
			if(hello == null)
				throw new RuntimeException();

			System.out.println("Hello's Credentials");
			orbasec.corba.CredUtil.dumpCredentials(current.get_target_credentials(hello));
			
			Hello hello_2 = null;
			// Uncomment to experiment with Object Invocation Policies
/* *
			//
			// Modify Object Invocation Policies
			//
			policies = new org.omg.CORBA.Policy[1];

			// QOP Policy:  NoProtection should raise a NO_RESOURCES
			//              exception if the server shut off NoProtection
			//              in accepting_options_supported.
			policies[0] = 
			current.create_qop_policy(org.omg.Security.QOP.SecQOPIntegrity);
// 			policies[0] = 
// 			current.create_qop_policy(org.omg.Security.QOP.SecQOPNoProtection);

			//
			// Set the policy on the hello_2 Object reference
			//
			System.out.println("Creating hello_2");
			hello_2 =
				HelloHelper.narrow(
					hello._set_policy_overrides(
						policies, 
						org.omg.CORBA.SetOverrideType.ADD_OVERRIDE));
/* */


    		//
    		// Main loop
    		//
    		System.out.println("Enter 'h' for hello or 'x' for exit:");
    		try {
				String input;
				do {
					System.out.print("> ");
					DataInputStream dataIn = new DataInputStream(System.in);
					BufferedReader in =
	    				new BufferedReader(new InputStreamReader(dataIn));
					input = in.readLine();

					if(input.equals("h")) {
						hello.hello();
						if ( hello_2 != null ){
							System.out.println( "Invoking hello() on hello_2" );
							hello_2.hello();
						}
	    			}
				} while(!input.equals("x"));
			} catch(IOException ex) {
				System.err.println("Can't read from `" +
					   ex.getMessage() + "'");
				System.exit(1);
			}

			System.exit(0);
    	} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
    	}
    }
}
