//tabstop=4
//*****************************************************************************/
// Project: ORBAsec
//
// -------------------------------------------------------------------------
// Copyright (C) 1998 Polar Humenn
//                    All rights reserved.
// -------------------------------------------------------------------------
// $Id$
//*****************************************************************************/

package hello;

import org.omg.CORBA.*;
import java.io.*;
import orbasec.corba.Opaque;
import orbasec.corba.MechUtil;

public class HelloServer
{

	// Lets get the authority of our certificate.
	static void
	setupTrustedAuthority(org.omg.SecurityLevel2.Current current)
	{
		try {
		    iaik.utils.KeyAndCertificate kac = null;
			// Use iaik to read the file.
            kac = new iaik.utils.KeyAndCertificate("server.pem");
			java.security.cert.X509Certificate[] chain =
				kac.getCertificateChain();
			orbasec.SecLev2.TrustedAuthority[] auths = 
				new orbasec.SecLev2.TrustedAuthority[1];

			byte[] encoded_name = 
				Opaque.encodeDirectoryName(
					((iaik.asn1.structures.Name)
						chain[chain.length-1].getIssuerDN()).getEncoded())
							.getEncoding();
			// We set up our trusted length to be that of the 
			// chain length-1. The subtraction for one is because
			// a distance of 0 means that the authority is the 
			// prinicpal and it must sign its own certificate.
			// A distance of one, means that the authority certificate
			// must sign the principals certificate, and so on.
			auths[0] = new orbasec.SecLev2.TrustedAuthority(
								"SSL",encoded_name, chain.length-1);
			
			// We will use this Trusted Authority for all our
			// clients and servers that we connect to, as well as
			// our verification of own principals.
			orbasec.SecLev2.TrustedAuthorityPolicyContent authpol = 
				new orbasec.SecLev2.TrustedAuthorityPolicyContent(
					auths, auths, auths);
					
			orbasec.SecLev2.TrustedAuthorityPolicy p =
				orbasec.SL2.create_trusted_authority_policy(authpol);

			org.omg.CORBA.Policy[] policies = 
				new org.omg.CORBA.Policy[1];

			policies[0] = p;

			((orbasec.SecLev2.Current)current)
				.set_overrides(policies, org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
		} catch (Exception e) {
			e.printStackTrace();
			throw new org.omg.CORBA.INITIALIZE(e.toString());
		}
	}
	
	public static void main(String args[])
	{
		try {
			//
			// ORB, BOA, and SL2 initialization
			//
			java.util.Properties properties = new java.util.Properties();
			orbasec.SL2.init_with_boa( 
				args, properties, properties );
			ORB orb = orbasec.SL2.orb();
			BOA boa = orbasec.SL2.boa();

    		// 
    		// Get SecurityLevel2::Current from ORB
    		// 
    		org.omg.SecurityLevel2.Current current = 
			org.omg.SecurityLevel2.CurrentHelper.narrow(
	    		orb.resolve_initial_references("SecurityCurrent"));

			setupTrustedAuthority(current);
			
    		//
    		// Authenticate using PrincipalAuthenticator
    		//
    		org.omg.SecurityLevel2.PrincipalAuthenticator pa;
    		pa = current.principal_authenticator();

			String  mechanism = MechUtil.SSL_DH_DSS_EXPORT_MECH;
			int	    method    = 0;
			byte	security_name[] = new byte[0]; // assigned from cert
			byte	auth_data[] =
			        ("cert_file=server.pem\n" +
					 "passphrase=\"blahblah\"\n" +
					 "enable_server=true\n" ).getBytes();
			org.omg.Security.SecAttribute privileges[] =
				new org.omg.Security.SecAttribute[0];
			org.omg.SecurityLevel2.CredentialsHolder creds_holder =
				new org.omg.SecurityLevel2.CredentialsHolder();
			org.omg.Security.OpaqueHolder
			 continuation_data =
				new org.omg.Security.OpaqueHolder(),
			 auth_specific_data =
				new org.omg.Security.OpaqueHolder();

			pa.authenticate(
				method,
				mechanism,
				security_name,
				auth_data,
				privileges,
				creds_holder,
				continuation_data,
				auth_specific_data
			);

			//
			// Print credentials to screen
			//
			org.omg.SecurityLevel2.Credentials[] credlist = 
				current.own_credentials();
			System.out.println("Own Credentials:");
			for( int i = 0;  i < credlist.length;  i++ ) {	
				   orbasec.corba.CredUtil.dumpCredentials(credlist[i]);
			}

			// Turn on EstablishTrustInClient requirement
			creds_holder.value.accepting_options_required( (short) 
				(creds_holder.value.accepting_options_required() |
				 org.omg.Security.EstablishTrustInClient.value));
			

			//
			// Create implementation object
			//
			Hello_impl p = new Hello_impl();
			p.orb        = orb;

			//
			// Save reference
			//
			try {
				String ref = orb.object_to_string(p);
				String refFile = "Hello.ref";
				FileOutputStream file = new FileOutputStream(refFile);
				PrintWriter out = new PrintWriter(file);
				out.println(ref);
				out.flush();
				file.close();
			} catch(IOException ex) {
				System.err.println("Can't write to `" +
						   ex.getMessage() + "'");
				System.exit(1);
			}

			//
			// Run implementation
			//
			boa.impl_is_ready(null);

			System.exit(0);
		} catch(Exception ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
			System.exit(1);
		}
	}
}
